@@ -119,14 +119,17 @@ func endpointHandler(req *request.Request) {
 // CreateBucket, ListRegionalBuckets which must resolve endpoint to s3-outposts.{region}.amazonaws.com
 // with region as client region and signed by s3-control if an outpost id is provided.
 func updateRequestOutpostIDEndpoint(request *request.Request) {
-	serviceEndpointLabel := "s3-outposts."
 	cfgRegion := aws.StringValue(request.Config.Region)
 
-	// request url
-	request.HTTPRequest.URL.Host = serviceEndpointLabel + cfgRegion + ".amazonaws.com"
+	if !hasCustomEndpoint(request) {
+		serviceEndpointLabel := "s3-outposts."
 
-	// disable the host prefix for outpost access points
-	request.Config.DisableEndpointHostPrefix = aws.Bool(true)
+		// request url
+		request.HTTPRequest.URL.Host = serviceEndpointLabel + cfgRegion + ".amazonaws.com"
+
+		// disable the host prefix for outpost access points
+		request.Config.DisableEndpointHostPrefix = aws.Bool(true)
+	}
 
 	// signer redirection
 	request.ClientInfo.SigningName = "s3-outposts"
@@ -169,7 +172,7 @@ func updateRequestOutpostBucketEndpoint(req *request.Request, bucketResource arn
 func validateEndpointRequestResource(req *request.Request, resource arn.Resource) error {
 	resReq := s3shared.ResourceRequest{Request: req, Resource: resource}
 
-	if resReq.IsCrossPartition() {
+	if len(resReq.Request.ClientInfo.PartitionID) != 0 && resReq.IsCrossPartition() {
 		return s3shared.NewClientPartitionMismatchError(resource,
 			req.ClientInfo.PartitionID, aws.StringValue(req.Config.Region), nil)
 	}
@@ -179,10 +182,6 @@ func validateEndpointRequestResource(req *request.Request, resource arn.Resource
 			req.ClientInfo.PartitionID, aws.StringValue(req.Config.Region), nil)
 	}
 
-	if resReq.HasCustomEndpoint() {
-		return s3shared.NewInvalidARNWithCustomEndpointError(resource, nil)
-	}
-
 	// Accelerate not supported
 	if aws.BoolValue(req.Config.S3UseAccelerate) {
 		return s3shared.NewClientConfiguredForAccelerateError(resource,
@@ -203,8 +202,9 @@ func validateOutpostEndpoint(req *request.Request, resource arn.Resource) error
 	}
 
 	// resource configured with FIPS as region is not supported by outposts
-	if resReq.ResourceConfiguredForFIPS() {
-		return s3shared.NewInvalidARNWithFIPSError(resource, nil)
+	if resReq.UseFIPS() {
+		return s3shared.NewFIPSConfigurationError(resource, req.ClientInfo.PartitionID,
+			aws.StringValue(req.Config.Region), nil)
 	}
 
 	// DualStack not supported
